@@ -25,10 +25,6 @@
 
 #include <glib/gi18n-lib.h>
 
-#include <telepathy-glib/account-manager.h>
-#include <telepathy-glib/interfaces.h>
-#include <telepathy-glib/util.h>
-
 #include <folks/folks.h>
 #include <folks/folks-telepathy.h>
 
@@ -430,6 +426,7 @@ contact_finalize (GObject *object)
 
   g_clear_object (&priv->groups);
   g_free (priv->alias);
+  g_free (priv->logged_alias);
   g_free (priv->id);
   g_strfreev (priv->client_types);
 
